;//---------------------------------------------------------------------------
// GPGee - GNU Privacy Guard Explorer Extension
// GPGeeContextMenuExtensionImpl.cpp - Explorer extension COM code
// Copyright 2005, Kurt Fitzner <kfitzner@excelcia.org>
//---------------------------------------------------------------------------
// This file is part of GPGee.
//
// GPGee is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// GPGee is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// File Notes:
// The main COM server implementation is here.  This code sets up the
// Windows explorer context menu.  Also, through some black magic, the
// DLL's DllRegisterServer() call ends up calling the UpdateRegistry()
// method below.
//
// The basic framework for how to make an explorer extension is convered
// in a tutorial available from Borland's developer network web site at
// http://bdn.borland.com/article/0,1410,26650,00.html
// The article is titled "Context Menu Shell Extensions Using COM" - it was
// helpful though it infers that creating a submenu of the context menu is
// only possible using owner draw menu items, which is not true.
//---------------------------------------------------------------------------

// GPGEECONTEXTMENUEXTENSIONIMPL : Implementation of TGPGeeContextMenuExtensionImpl (CoClass: GPGeeContextMenuExtension, Interface: IGPGeeContextMenuExtension)

#include <vcl.h>
#pragma hdrstop


#include "GPGEECONTEXTMENUEXTENSIONIMPL.H"
#include "GPGeeSignEncrypt.h"
#include "GPGeeVerifyDecrypt.h"
#include "GPGeeConfigure.h"

/////////////////////////////////////////////////////////////////////////////
// TGPGeeContextMenuExtensionImpl

//---------------------------------------------------------------------------
// SLOW, Construction ahead.
//
TGPGeeContextMenuExtensionImpl::TGPGeeContextMenuExtensionImpl() {
  FileNames = NULL;
  FileNames = new TStringList;
}  // TGPGeeContextMenuExtensionImpl::TGPGeeContextMenuExtensionImpl()
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
//  End Construction Zone
//
TGPGeeContextMenuExtensionImpl::~TGPGeeContextMenuExtensionImpl() {
  if (FileNames)
    delete FileNames;
}  // TGPGeeContextMenuExtensionImpl::~TGPGeeContextMenuExtensionImpl()
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// (Un)Register the object
//
HRESULT WINAPI TGPGeeContextMenuExtensionImpl::UpdateRegistry(BOOL bRegister) {
  if (bRegister) {
    CreateRegKey("*\\shellex", "", "");
    CreateRegKey("*\\shellex\\ContextMenuHandlers", "", "");
    CreateRegKey("*\\shellex\\ContextMenuHandlers\\GPGee", "", Comobj::GUIDToString(GetObjectCLSID()));
    CreateRegKey("Directory", "", "");
    CreateRegKey("Directory\\shellex", "", "");
    CreateRegKey("Directory\\shellex\\ContextMenuHandlers", "", "");
    CreateRegKey("Directory\\shellex\\ContextMenuHandlers\\GPGee", "", Comobj::GUIDToString(GetObjectCLSID()));
  } else {
    DeleteRegKey("*\\shellex\\ContextMenuHandlers\\GPGee");
    DeleteRegKey("Directory\\shellex\\ContextMenuHandlers\\GPGee");
  }  // if (bRegister)
  TTypedComServerRegistrarT<TGPGeeContextMenuExtensionImpl>
  regObj(GetObjectCLSID(), GetProgID(), GetDescription());
  return regObj.UpdateRegistry(bRegister);
}  // HRESULT WINAPI TGPGeeContextMenuExtensionImpl::UpdateRegistry()
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
//
// IShellExtInit
//
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Initialize method for IShellExtInit
//
// Get the filenames of all the files the user is invoking the context menu over
// and store them in a TStringList
//
STDMETHODIMP TGPGeeContextMenuExtensionImpl::Initialize(LPCITEMIDLIST pidlFolder, LPDATAOBJECT lpdobj, HKEY hkeyProgID)
{

  HDROP hDrop;
  HRESULT hResult;
  FORMATETC fmtEtc;
  STGMEDIUM medium;
  UINT uFileCount;
  char *FileName;
  AnsiString Extension;

  fmtEtc.cfFormat = CF_HDROP;
  fmtEtc.ptd      = NULL;
  fmtEtc.dwAspect = DVASPECT_CONTENT;
  fmtEtc.lindex   = -1;
  fmtEtc.tymed    = TYMED_HGLOBAL;

  medium.tymed    = TYMED_HGLOBAL;

  if ((hResult = lpdobj->GetData(&fmtEtc,&medium)) < 0)
    return E_INVALIDARG;

  if ((hDrop = (HDROP)GlobalLock(medium.hGlobal)) == NULL)
    return E_INVALIDARG;

  FileName = new char[MAX_PATH];
  uFileCount = DragQueryFile(hDrop, 0xFFFFFFFF, NULL, 0);
  AllFilesAreGPG = true;
  if (uFileCount) {
    FileNames->Clear();
    for (UINT n=0; n < uFileCount; n++) {
      DragQueryFile(hDrop, n, FileName, MAX_PATH);
      FileNames->Add(String(FileName));
      Extension = ExtractFileExt(String(FileName));
      if (Extension.AnsiCompareIC(".gpg") != 0 && Extension.AnsiCompareIC(".asc") != 0 && Extension.AnsiCompareIC(".sig") != 0)
        AllFilesAreGPG = false;
    }  // for (UINT n=0; n < uFileCount; n++)
    hResult = S_OK;
  } else {
    hResult = E_INVALIDARG;
  }  // if (uFileCount)

  GlobalUnlock(medium.hGlobal);
  ReleaseStgMedium(&medium);
  delete FileName;

  return hResult;
}  // TGPGeeContextMenuExtensionImpl::Initialize()
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
//
// IContextMenu
//
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// GetCommandString()
// Set the help string associated with our menu entries
// FIXME: Put something here
//
STDMETHODIMP TGPGeeContextMenuExtensionImpl::GetCommandString(UINT idCmd, UINT uFlags, UINT* pwReserved, LPSTR pszName, UINT cchMax)
{
/*  HRESULT hResult;
  USES_CONVERSION;*/

  return S_OK;
}  // STDMETHODIMP TGPGeeContextMenuExtensionImpl::GetCommandString()
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// QueryContextMenu()
// Add our items to the context menu
//
STDMETHODIMP TGPGeeContextMenuExtensionImpl::QueryContextMenu(HMENU hMenu, UINT indexMenu, UINT idCmdFirst, UINT idCmdLast, UINT uFlags)
{
  HMENU hPopup;

  if (uFlags & CMF_DEFAULTONLY || !FileNames->Count)
    return MAKE_HRESULT(SEVERITY_SUCCESS, FACILITY_NULL, 0);

  hPopup = CreatePopupMenu();
  if (hPopup) {
    if (AllFilesAreGPG) {
      InsertMenu(hPopup, GPGEE_VERIFYDECRYPT, MF_BYPOSITION|MF_STRING, idCmdFirst++, "Verify/Decrypt");
      InsertMenu(hPopup, GPGEE_SEP1,          MF_SEPARATOR|MF_BYPOSITION, idCmdFirst++, NULL);
      InsertMenu(hPopup, GPGEE_DCONFIGURE,    MF_BYPOSITION|MF_STRING, idCmdFirst++, "Configure GPGee");
    } else {
      InsertMenu(hPopup, GPGEE_SIGNENCRYPT,   MF_BYPOSITION|MF_STRING, idCmdFirst++, "Sign and Encrypt");
      InsertMenu(hPopup, GPGEE_SIGN,          MF_BYPOSITION|MF_STRING, idCmdFirst++, "Sign");
      InsertMenu(hPopup, GPGEE_ENCRYPT,       MF_BYPOSITION|MF_STRING, idCmdFirst++, "Encrypt");
      InsertMenu(hPopup, GPGEE_SEP2,          MF_SEPARATOR|MF_BYPOSITION, idCmdFirst++, NULL);
      InsertMenu(hPopup, GPGEE_ECONFIGURE,    MF_BYPOSITION|MF_STRING, idCmdFirst++, "Configure GPGee");
    }  // if (AllFilesAreGPG)
    // InsertMenu(hPopup, GPGEE_VERIFY,      MF_BYPOSITION|MF_STRING, idCmdFirst++, "Decrypt/Verify");
  }  // if (hPopup)

  InsertMenu(hMenu, indexMenu++, MF_SEPARATOR|MF_BYPOSITION, 0, NULL);
  InsertMenu(hMenu, indexMenu, MF_STRING|MF_BYPOSITION|MF_POPUP, (UINT)hPopup, "GPGee");

  return MAKE_HRESULT(SEVERITY_SUCCESS, FACILITY_NULL, 5);
}  // STDMETHODIMP TGPGeeContextMenuExtensionImpl::QueryContextMenu()
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// InvokeCommand
// Handler for the context menu items
//
STDMETHODIMP TGPGeeContextMenuExtensionImpl::InvokeCommand(LPCMINVOKECOMMANDINFO lpici)
{
  WORD cmdId = LOWORD(lpici->lpVerb);
  HWND handle;
  if (HIWORD(lpici->lpVerb))
    return E_FAIL;

  if (AllFilesAreGPG) {
    switch(cmdId) {
      case GPGEE_VERIFYDECRYPT: HandleVerifyDecrypt(); break;
      case GPGEE_DCONFIGURE:    HandleConfigure(); break;
      default:
        return E_FAIL;
    }  // switch(cmdId))
  } else {
    switch(cmdId) {
      case GPGEE_SIGNENCRYPT:   HandleSignEncrypt(true, true);  break;
      case GPGEE_SIGN:          HandleSignEncrypt(true, false); break;
      case GPGEE_ENCRYPT:       HandleSignEncrypt(false, true); break;
      case GPGEE_ECONFIGURE:    HandleConfigure(); break;
      default:
        return E_FAIL;
    }  // switch(cmdId)
  }  // if (AllFilesAreGPG)

  return S_OK;
}  // STDMETHODIMP TGPGeeContextMenuExtensionImpl::InvokeCommand()
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Handle signing and encrypting - create the form and then pass the buck
//
void __fastcall TGPGeeContextMenuExtensionImpl::HandleSignEncrypt(bool bSign, bool bEncrypt)
{
  formGPGeeSignEncrypt = new TformGPGeeSignEncrypt(NULL);
  formGPGeeSignEncrypt->ShowModalWithDefaults(bSign, bEncrypt, FileNames);
  formGPGeeSignEncrypt->Release();
}  // __fastcall TGPGeeContextMenuExtensionImpl::HandleSignEncrypt(bool bSign, bool bEncrypt)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Handle verifying and decrypting
//
void __fastcall TGPGeeContextMenuExtensionImpl::HandleVerifyDecrypt(void)
{
  formGPGeeVerifyDecrypt = new TformGPGeeVerifyDecrypt(NULL);
  formGPGeeVerifyDecrypt->ShowModalWithDefaults(FileNames);
  formGPGeeVerifyDecrypt->Release();
}  // __fastcall TGPGeeContextMenuExtensionImpl::HandleSignEncrypt(bool bSign, bool bEncrypt)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Handle configuration of the extension
//
void __fastcall TGPGeeContextMenuExtensionImpl::HandleConfigure(void)
{
  formGPGeeConfigure = new TformGPGeeConfigure(NULL);
  formGPGeeConfigure->ShowModal();
  formGPGeeConfigure->Release();
}  // void __fastcall TGPGeeContextMenuExtensionImpl::HandleConfigure(void)
//---------------------------------------------------------------------------

