//---------------------------------------------------------------------------
// GPGee - GNU Privacy Guard Explorer Extension
// GPGeeUtility.cpp - Miscelaneous utility functions
// Copyright 2005, Kurt Fitzner <kfitzner@excelcia.org>
//---------------------------------------------------------------------------
// This file is part of GPGee.
//
// GPGee is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// GPGee is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
//---------------------------------------------------------------------------
#include <vcl.h>
#include <FileCtrl.hpp>
#pragma hdrstop

#include "gpgme/gpgme.h"
#include "GPGeeUtility.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Take a filename and reduce it so that it fits inside the space given on
// an edit box.
//
AnsiString __fastcall MinimizeName(AnsiString FileName, TEdit *Edit)
{
  TCanvas *FileNameCanvas = new TCanvas;
  FileNameCanvas->Handle = GetDC(GetActiveWindow());
  FileNameCanvas->Font->Assign(Edit->Font);
  Edit->Text = Filectrl::MinimizeName(FileName, FileNameCanvas, Edit->Width-4);
  ReleaseDC(GetActiveWindow(),FileNameCanvas->Handle);
  return Edit->Text;
}  // AnsiString __fastcall MinimizeName(AnsiString FileName, TControl *Control)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Take a filename and reduce it so that it fits inside the space given on
// an edit box.
//
AnsiString __fastcall MinimizeName(AnsiString FileName, TFont *Font, int Width)
{
  AnsiString retval;

  TCanvas *FileNameCanvas = new TCanvas;
  FileNameCanvas->Handle = GetDC(GetActiveWindow());
  FileNameCanvas->Font->Assign(Font);
  retval = Filectrl::MinimizeName(FileName, FileNameCanvas, Width);
  ReleaseDC(GetActiveWindow(),FileNameCanvas->Handle);
  return retval;
}  // AnsiString __fastcall MinimizeName(AnsiString FileName, TControl *Control)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Identify a GnuPG file
//
gpg_file_type __fastcall IdentifyFile(AnsiString FileName)
{
  AnsiString Extension = ExtractFileExt(FileName);
  gpg_file_type retval = GPG_UNKNOWN;

  if (Extension.AnsiCompareIC(".gpg") == 0)
    retval = GPG_MESSAGE;
  else if (Extension.AnsiCompareIC(".sig") == 0)
    retval = GPG_SIGNATURE;
  else if (Extension.AnsiCompareIC(".asc") == 0) {
    gpgme_pgptype_t pgptype;
    if (gpgme_file_get_pgptype(FileName.c_str(), &pgptype) == GPGME_No_Error)
      switch (pgptype) {
        case GPGME_TYPE_MESSAGE:
        case GPGME_TYPE_CLEARSIG:
          retval = GPG_MESSAGE;
          break;
        case GPGME_TYPE_SIG:
          retval = GPG_SIGNATURE;
          break;
        default:
          retval = GPG_UNKNOWN;
      }  // switch (pgptype)
  }  // else if (Extension.AnsiCompareIC(".asc") == 0)
  return retval;
}  // int __fastcall IdentifyFile(AnsiString FileName)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Convert GnuPG dates to TDateTime
//
TDateTime __fastcall GPGToDateTime(unsigned long GPGDate) {
  TIME_ZONE_INFORMATION tzInfo;
  DWORD tzResult;
  static double tzBias = -1.0;

  // GnuPG returns timestamps in GMT - we convert that to local time here, so we need to get the time zone info
  if (tzBias == -1.0) {
    tzResult = GetTimeZoneInformation(&tzInfo);
    if (tzResult == TIME_ZONE_ID_STANDARD)
      tzBias = ((double)tzInfo.Bias + (double)tzInfo.StandardBias) / 1440.0;
    else if (tzResult == TIME_ZONE_ID_DAYLIGHT)
      tzBias = ((double)tzInfo.Bias + (double)tzInfo.DaylightBias) / 1440.0;
    else
      tzBias = 0.0;
  }  // if (tzBias = -1.0)
  return (double)GPGDate / (24.0 * 60.0 * 60.0) + UnixDateDelta - tzBias;
}  // TDateTime __fastcall GPGToDateTime(unsigned long GPGDate)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Return a short string description for a GnuPG public key id number
//
AnsiString __fastcall KeyTypeToString(gpgme_pk_cipher_t KeyType)
{
  AnsiString retval;
  switch (KeyType) {
    case GPGME_PK_RSA:
      retval = "RSA"; break;
    case GPGME_PK_RSA_E:
      retval = "RSAe"; break;
    case GPGME_PK_RSA_S:
      retval = "RSAs"; break;
    case GPGME_PK_ELG_E:
      retval = "ELG"; break;
    case GPGME_PK_DSA:
      retval = "DSA"; break;
    case GPGME_PK_ELG_ES:
      retval = "ELGse"; break;
  }  // switch (KeyType)
  return retval;
}  // AnsiString __fastcall KeyTypeToString(gpgme_pk_cipher_t KeyType)
//---------------------------------------------------------------------------

